#!/bin/sh

### BEGIN INIT INFO
# Provides:          amc
# Required-Start:    $remote_fs $syslog
# Required-Stop:     $remote_fs $syslog
# Default-Start:     2 3 4 5
# Default-Stop:      0 1 6
# Short-Description: Start daemon at boot time
# Description:       Enable service provided by daemon.
### END INIT INFO

PROJECT="/opt/amc"
PIDFILE="/tmp/amc.pid"
CRONJOB="${PROJECT}/bin/start_amc_cron.sh"
GUNICORN="${PROJECT}/bin/gunicorn"
CONFIG="/etc/amc/config/gunicorn_config.py"
AMCCONFIG="/etc/amc/config/amc.cfg"

inplace_edit_file(){
    TMP_FILE=`mktemp /tmp/config.XXXXXXXXXX`
    sed -e "$1" $2 > $TMP_FILE
    mv $TMP_FILE $2
}


# This is to check whether certificate file path is
# provided or no

cert_file=$(awk '/^certfile/{print $3}' $AMCCONFIG)
key_file=$(awk '/^keyfile/{print $3}' $AMCCONFIG)

cert_file_path="${cert_file%\"}"
cert_file_path="${cert_file_path#\"}"

key_file_path="${key_file%\"}"
key_file_path="${key_file_path#\"}"

if [ ! -e "$cert_file_path" ]; then
  inplace_edit_file "/^certfile/d" $CONFIG
elif [ "$(awk -F '\n' '/^certfile/{print $1}' $CONFIG)" = "" ]; then
  echo "certfile = \"<certificate_file_path>\"" >> $CONFIG
fi
if [ ! -e "$key_file_path" ]; then
  inplace_edit_file "/^keyfile/d" $CONFIG
elif [ "$(awk -F '\n' '/^keyfile/{print $1}' $CONFIG)" = "" ]; then
  echo "keyfile = \"<key_file_path>\"" >> $CONFIG
fi

cert_file="certfile = $cert_file"
key_file="keyfile = $key_file"

inplace_edit_file "s|^certfile =.*|$cert_file|g" $CONFIG
inplace_edit_file "s|^keyfile =.*|$key_file|g" $CONFIG

CMD="${GUNICORN} --daemon --config=${CONFIG} flaskapp:app"
port="8081"
get_port() {
  while read -r line           
  do              
    if echo "$line" | grep -q "bind"; then     
      data=${line##*=}
      port=${data##*:}
      port=${port%?}
    fi
  done </etc/amc/config/gunicorn_config.py
}
check_amc_status(){
    status=1
    if [ -z "`ps aux | grep amc | grep -v grep | grep gunicorn | grep master`" ]; then
        status=0
    fi
    echo $status
}

check_cronjob_status(){
    status=1
    if [ -z "`ps aux | grep ${CRONJOB} | grep -v grep`" ]; then
        status=0
    fi
    echo $status
}

start_amc(){
  status=$(check_amc_status)
  if [ $status -eq 0 ] ; then
      get_port
      port_check=$(netstat -ln | grep ':$port ' | grep LISTEN)
      if [ "$port_check" = "" ]
      then
        rm -f $PIDFILE
        $CMD
        if [ $? = 0 ]; then
          echo "AMC is started."
        else
          echo "AMC ${red}failed${default}."
        fi
      else
        echo "Port 8081 is being used, please unlock the port and try to start it again."
        exit 1
      fi
  else
    echo "AMC is running."
    exit 1
  fi
}

stop_amc(){
  status=$(check_amc_status)
  if [ $status -eq 0 ] ; then
    echo "AMC is ${red}not running${default}."
  else
    kill -9 `cat $PIDFILE`
    kill -9 `ps aux | grep amc | grep -v grep | grep worker | awk '{print $2}'`
    $(remove_cron) 
    if [ $? = 0 ]; then
      echo "AMC is stopped."
      rm -f $PIDFILE
    else
      echo "AMC could not be stopped."
    fi
  fi
}

amc_status(){
  status=$(check_amc_status)
  if [ $status -eq 0 ] ; then
    echo "AMC is not running."
  else
    echo "AMC is running."
  fi
}

start_cron(){
    status=$(check_cronjob_status)
    if [ $status -eq 0 ]; then
       sh $CRONJOB
    fi
}

remove_cron(){
    echo 1 > /opt/amc/stop_signal
}

# Carry out specific functions when asked to by the system
case "$1" in
    start)
        python_version=$(python -c 'import sys; print int(sys.version_info[0:2] > (2, 5) and sys.version_info[0:2] < (3,))')
        if [ $python_version -ne 0 ]; then
            echo "Starting AMC...."
            output=$(start_amc)
            echo "$output"
            sleep 1
            $(start_cron) &
        else
            echo "Unable to start the AMC, unsupported Python version found. Please update the Python version to 2.6 or 2.7"
        fi
    ;;
    stop)
	    echo "Stopping AMC...."
	    output=$(stop_amc)
            $(remove_cron)
	    echo "$output"
    ;;
    status)
            echo "Retrieving AMC status...."
	    output=$(amc_status)
            echo "$output"
    ;;
    restart)
	    echo "Restarting AMC...."
	    output="$(stop_amc)"
            echo "$output"
            sleep 1
            python_version=$(python -c 'import sys; print int(sys.version_info[0:2] > (2, 5) and sys.version_info[0:2] < (3,))')
            if [ $python_version -ne 0 ]; then
                output=$(start_amc)
                sleep 1
                $(start_cron) &
                echo "$output"   
            else
                echo "Unable to restart the AMC, unsupported Python version found. Please update the Python version to 2.6 or 2.7"
            fi
    ;;
    --help)
           cat ${PROJECT}/README
    ;;
    --version)
           cat ${PROJECT}/amc_version
           echo
    ;;
    *)
        echo "Usage: /etc/init.d/amc {start|stop|restart|status}"
        exit 1
    ;;
        
esac

exit 0

